/*****************************************************************************/
/* N.A.D.A.R. - A network tank battle game.                                  */
/*                                                                           */
/* N.A.D.A.R. Copyright (c) 2000 Sakai Hiroaki.                              */
/* All Rights Reserved.                                                      */
/*                                                                           */
/* N.A.D.A.R. is a network tank battle game. You get into a tank,            */
/* operate it, and destroy other tanks to survive!                           */
/*****************************************************************************/

/*****************************************************************************/
/* Game 饹(Υ饹)Υ饤֥                             */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                   */
/*****************************************************************************/

#include "GameP.h"

/*****************************************************************************/
/* ؿ                                                                */
/*****************************************************************************/

/*===========================================================================*/
/* ѿμ                                                    */
/*===========================================================================*/

Path GameGetRequest(Game game) { return (game->request); }
ObjList GameGetPlayerList(Game game) { return (game->player_list); }

/*===========================================================================*/
/* ४СȽ                                                      */
/*===========================================================================*/

static int GameIsOver(Game game)
{
  return (0);
}

/*===========================================================================*/
/* ֥ȤȺ                                                  */
/*===========================================================================*/

/*---------------------------------------------------------------------------*/
/* ֥Ȥ                                                        */
/*---------------------------------------------------------------------------*/

Game GameCreate(char * hostname, unsigned short int port)
{
  Game game;

  InitializeRandom(); /*  */

  game = (Game)malloc(sizeof(_Game));
  if (game == NULL) ErrorExit("Cannot allocate memory for game.");

  game->stage = NULL;

  game->player_list = ObjList_Create();
  if (game->player_list == NULL) ErrorExit("Cannot create game->player_list.");

  game->request = PathCreateToAcceptRequestsFromClients(hostname, port, 5);
  if (game->request == NULL) ErrorExit("Cannot create game->request.");

  game->stage_number = 0;

  return (game);
}

/*---------------------------------------------------------------------------*/
/* ֥Ȥκ                                                        */
/*---------------------------------------------------------------------------*/

Game GameDestroy(Game game)
{
  if (game == NULL) return (NULL);

  if (game->stage) StageDestroy(game->stage);

  /* tank_list  game->stage ȶͭƤΤǡ*/
  /* game->stage ˳뤳 */
  if (game->player_list) ObjList_Destroy(game->player_list);

  if (game->request) PathDestroy(game->request);

  free(game);

  return (NULL);
}

/*---------------------------------------------------------------------------*/
/* Υ                                                          */
/*---------------------------------------------------------------------------*/

int GameStart(Game game, int silent)
{
  Path path;
  char player_name[100];
  unsigned short int command;
  unsigned short int length;
  Player player;
  int i;
  ObjListData current;
  int ret;

  if (!silent) {
    printf("\n"
	   "                                 N.A.D.A.R.\n"
	   "                        A network tank battle game.\n"
	   "\n"
	   "               N.A.D.A.R. Copyright (c) 2000 Sakai Hiroaki.\n"
	   "                            All Rights Reserved.\n"
	   "\n"
	   "N.A.D.A.R. is a client-server model program. "
	   "This is N.A.D.A.R.\'s server.\n"
	   "\n"
	   "Starting N.A.D.A.R. server.\n"
	   "Start N.A.D.A.R. clients as\n"
	   "\n"
	   "  %% nadar -s [Server\'s hostname] -p [Player\'s name]\n"
	   "\n"
	   "If playing speed is very slow, "
	   "run clients as below with size option.\n"
	   "\n"
	   "  %% nadar -s [Server's hostname] -p [Player\'s name] -size 30\n"
	   "\n");
  }

  while (!GameIsOver(game)) {

    if (ObjList_GetLength(game->player_list) < 1)
      game->stage_number = 0;

    /* 饤Ȥ³׵μդ */
    while ((ObjList_GetLength(game->player_list) < PLAYER_NUMBER_MIN)
	   || PathCheckRecvState(game->request)) {
      i = 1;
      while(!PathCheckRecvState(game->request)) {
	if (!silent && !(i % 5)) printf("Waiting accept from clients...\n");
	i++;
	sleep(1);
      }

      path = PathCreateToCommunicateWithClient(game->request,
					       SERVER_SEND_BUFFER_SIZE,
					       SERVER_RECV_BUFFER_SIZE);

      while (!PathCheckRecvState(path)) { usleep(10000); }
      ret = PathCheckHeader(path, &command, &length);

      if (ret && (command == JOIN_REQUEST)) {

	PathRecvStringData(path, &command, player_name, 100);

	player = PlayerCreate(player_name, path, PLAYER_WAIT);
	ObjList_InsertObjToEnd(game->player_list, player,
			       (ObjDestructor)PlayerDestroy);

	PathSendNoneData(PlayerGetPath(player), JOIN_ACCEPT);
	PathFlushSendBuffer(PlayerGetPath(player));
      } else {
	path = PathDestroy(path);
      }

    }

    /* Wait ֤ˤʤäƤ Player  Active ˤ */
    /* Destroy ֤ Player  */

    for (current = ObjList_GetStart(game->player_list);
	 !ObjList_IsEndEdge(game->player_list, current);
	 current = ObjListData_GetNext(current)) {
      player = (Player)ObjListData_GetObj(current);
      switch (PlayerGetState(player)) {
      case PLAYER_WAIT :
	PlayerSetState(player, PLAYER_ACTIVE);
	break;
      case PLAYER_ACTIVE :
	break;
      case PLAYER_DESTROY :
	current = ObjList_DeleteObjToPrev(game->player_list, current);
	break;
      }
    }

    game->stage = StageCreate(game->stage_number, game->player_list);
    if (game->stage == NULL) ErrorExit("Cannot create game->stage.");

    StageStartAsServer(game->stage, game->player_list, game->request);

    game->stage = StageDestroy(game->stage);

    (game->stage_number)++;
  }

  return (0);
}

/*****************************************************************************/
/* ޤ                                                                  */
/*****************************************************************************/

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/
